﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Windows.Forms;
using DoenaSoft.DVDProfiler.DVDProfilerXML.Version381;

namespace DVD_Profiler_4._0
{
    public partial class MainForm : Form
    {
        public MainForm()
        {
            InitializeComponent();
        }

        private void OnOpenCollectionxmlToolStripMenuItemClick(Object sender, EventArgs e)
        {
            String fileName;
            Collection collection;

            using (OpenFileDialog ofd = new OpenFileDialog())
            {
                ofd.FileName = "collection.xml";
                ofd.Filter = "collection.xml|*.xml";
                ofd.CheckFileExists = true;
                ofd.Multiselect = false;
                ofd.RestoreDirectory = true;
                if (ofd.ShowDialog() == DialogResult.OK)
                {
                    fileName = ofd.FileName;
                }
                else
                {
                    return;
                }
            }
            try
            {
                collection = Collection.Deserialize(fileName);
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            if ((collection.DVDList != null) && (collection.DVDList.Length > 0))
            {
                this.ClearScreen();
                this.ImportProfile(collection.DVDList[0]);
            }
            else
            {
                MessageBox.Show("No DVD in collection!", "Warning", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                return;
            }
        }

        private void ImportProfile(DVD dvd)
        {
            ImportAudio(dvd);
            ImportSubtitles(dvd);
            ImportFeatures(dvd);
            MessageBox.Show("\"" + dvd.Title + "\" succesfully imported.", "Success", MessageBoxButtons.OK, MessageBoxIcon.Information);
        }

        private void ImportFeatures(DVD dvd)
        {
            if (dvd.Features != null)
            {
                List<String> availableFeatures;
                List<String> profileFeatures;
                String feature;

                availableFeatures = new List<String>(this.AvailableFeaturesListBox.Items.Cast<String>());
                profileFeatures = new List<String>();
                if (dvd.Features.BDLive)
                {
                    feature = "BD-Live";
                    profileFeatures.Add(feature);
                    availableFeatures.Remove(feature);
                }
                if (dvd.Features.BonusTrailers)
                {
                    feature = "Bonus Trailers";
                    profileFeatures.Add(feature);
                    availableFeatures.Remove(feature);
                }
                if (dvd.Features.ClosedCaptioned)
                {
                    Int32 index;
                    DataGridViewRow row;

                    index = this.SubtitleGrid.Rows.Add();
                    row = this.SubtitleGrid.Rows[index];
                    row.Cells[0].Value = "Main Feature";
                    row.Cells[1].Value = "Close Captioned (CC)";
                }
                if (dvd.Features.Commentary)
                {
                    //
                }
                if (dvd.Features.DeletedScenes)
                {
                    feature = "Deleted Scenes";
                    profileFeatures.Add(feature);
                    availableFeatures.Remove(feature);
                }
                if (dvd.Features.DigitalCopy)
                {
                    feature = "Digital Copy";
                    profileFeatures.Add(feature);
                    availableFeatures.Remove(feature);
                }
                if (dvd.Features.DVDROMContent)
                {
                    feature = "DVD-ROM Content";
                    profileFeatures.Add(feature);
                    availableFeatures.Remove(feature);
                }
                if (dvd.Features.Game)
                {
                    feature = "Interactive Games";
                    profileFeatures.Add(feature);
                    availableFeatures.Remove(feature);
                }
                if (dvd.Features.Interviews)
                {
                    feature = "Interviews";
                    profileFeatures.Add(feature);
                    availableFeatures.Remove(feature);
                }
                if (dvd.Features.MakingOf)
                {
                    feature = "Featurettes";
                    profileFeatures.Add(feature);
                    availableFeatures.Remove(feature);
                }
                if (dvd.Features.MultiAngle)
                {
                    feature = "Multi-Angle";
                    profileFeatures.Add(feature);
                    availableFeatures.Remove(feature);
                }
                if (dvd.Features.MusicVideos)
                {
                    feature = "Music Videos";
                    profileFeatures.Add(feature);
                    availableFeatures.Remove(feature);
                }
                if (dvd.Features.Outtakes)
                {
                    feature = "Outtakes/Bloopers";
                    profileFeatures.Add(feature);
                    availableFeatures.Remove(feature);
                }
                if (dvd.Features.PhotoGallery)
                {
                    feature = "Gallery";
                    profileFeatures.Add(feature);
                    availableFeatures.Remove(feature);
                }
                if (dvd.Features.PIP)
                {
                    feature = "Picture-in-Picture";
                    profileFeatures.Add(feature);
                    availableFeatures.Remove(feature);
                }
                if (dvd.Features.ProductionNotes)
                {
                    feature = "Production Notes/Bios";
                    profileFeatures.Add(feature);
                    availableFeatures.Remove(feature);
                }
                if (dvd.Features.SceneAccess)
                {
                    feature = "Scene Access";
                    profileFeatures.Add(feature);
                    availableFeatures.Remove(feature);
                }
                if (dvd.Features.StoryboardComparisons)
                {
                    feature = "Storyboard Comparison";
                    profileFeatures.Add(feature);
                    availableFeatures.Remove(feature);
                }
                if (dvd.Features.THXCertified)
                {
                    feature = "THX Certified";
                    profileFeatures.Add(feature);
                    availableFeatures.Remove(feature);
                }
                if (dvd.Features.Trailer)
                {
                    feature = "Feature Trailers";
                    profileFeatures.Add(feature);
                    availableFeatures.Remove(feature);
                }
                availableFeatures.Sort();
                profileFeatures.Sort();
                this.AvailableFeaturesListBox.Items.Clear();
                this.AvailableFeaturesListBox.Items.AddRange(availableFeatures.ToArray());
                this.ProfileFeatureListBox.Items.AddRange(profileFeatures.ToArray());
                ImportOtherFeatures(dvd.Features.OtherFeatures);
            }
        }

        private void ImportOtherFeatures(String otherFeatures)
        {
            if (String.IsNullOrEmpty(otherFeatures) == false)
            {
                String[] split;

                split = otherFeatures.Replace(Environment.NewLine, ",").Split(',');
                foreach (String otherFeature in split)
                {
                    String temp;

                    temp = otherFeature.Trim();
                    if (String.IsNullOrEmpty(temp) == false)
                    {
                        this.ProfileOtherFeatureListBox.Items.Add(temp);
                    }
                }
            }
        }

        private void ImportSubtitles(DVD dvd)
        {
            if ((dvd.SubtitleList != null) && (dvd.SubtitleList.Length > 0))
            {
                foreach (String subtitle in dvd.SubtitleList)
                {
                    Int32 index;
                    DataGridViewRow row;

                    index = this.SubtitleGrid.Rows.Add();
                    row = this.SubtitleGrid.Rows[index];
                    row.Cells[0].Value = "Main Feature";
                    switch (subtitle)
                    {
                        case ("Commentary"):
                            {
                                row.Cells[1].Value = subtitle;
                                break;
                            }
                        case ("Trivia"):
                            {
                                row.Cells[1].Value = subtitle;
                                break;
                            }
                        default:
                            {
                                row.Cells[2].Value = subtitle;
                                break;
                            }
                    }
                }
            }
        }

        private void ImportAudio(DVD dvd)
        {
            if ((dvd.AudioList != null) && (dvd.AudioList.Length > 0))
            {
                foreach (AudioTrack track in dvd.AudioList)
                {
                    Int32 index;
                    DataGridViewRow row;

                    index = this.AudioGrid.Rows.Add();
                    row = this.AudioGrid.Rows[index];
                    row.Cells[0].Value = "Main Feature";
                    switch (track.Content)
                    {
                        case ("Audio Descriptive"):
                            {
                                row.Cells[1].Value = track.Content;
                                break;
                            }
                        case ("Commentary"):
                            {
                                row.Cells[1].Value = track.Content;
                                break;
                            }
                        case ("Special Effects"):
                            {
                                row.Cells[1].Value = track.Content;
                                break;
                            }
                        default:
                            {
                                row.Cells[2].Value = track.Content;
                                break;
                            }
                    }
                }
            }
        }

        private void ClearScreen()
        {
            this.AudioGrid.Rows.Clear();
            this.SubtitleGrid.Rows.Clear();
            while (this.ProfileFeatureListBox.Items.Count > 0)
            {
                this.ProfileFeatureListBox.SelectedIndex = 0;
                this.OnRemoveFeatureButtonClick(this, EventArgs.Empty);
            }
            this.ProfileOtherFeatureListBox.Items.Clear();
            this.ProfileOtherFeatureTextBox.Text = String.Empty;
        }

        private void OnAvailableFeaturesListBoxSelectedIndexChanged(Object sender, EventArgs e)
        {
            this.AddFeatureButton.Enabled = (AvailableFeaturesListBox.SelectedIndex != -1);
        }

        private void OnProfileFeatureListBoxSelectedIndexChanged(Object sender, EventArgs e)
        {
            this.RemoveFeatureButton.Enabled = (ProfileFeatureListBox.SelectedIndex != -1);
        }

        private void OnOtherFeatureTextBoxTextChanged(Object sender, EventArgs e)
        {
            this.AddOtherFeatureButton.Enabled = (String.IsNullOrEmpty(this.ProfileOtherFeatureTextBox.Text) == false);
        }

        private void OnAddFeatureButtonClick(Object sender, EventArgs e)
        {
            Int32 selectedIndex;
            String feature;

            selectedIndex = this.AvailableFeaturesListBox.SelectedIndex;
            feature = this.AvailableFeaturesListBox.Items[selectedIndex].ToString();
            AddFeature(feature, this.ProfileFeatureListBox.Items);
            this.AvailableFeaturesListBox.Items.RemoveAt(selectedIndex);
        }

        private static void AddFeature(String feature, ListBox.ObjectCollection items)
        {
            List<String> features;

            features = new List<String>(items.Cast<String>());
            features.Add(feature);
            features.Sort();
            items.Clear();
            items.AddRange(features.ToArray());
        }

        private void OnRemoveFeatureButtonClick(Object sender, EventArgs e)
        {
            Int32 selectedIndex;
            String feature;

            selectedIndex = this.ProfileFeatureListBox.SelectedIndex;
            feature = this.ProfileFeatureListBox.Items[selectedIndex].ToString();
            AddFeature(feature, this.AvailableFeaturesListBox.Items);
            this.ProfileFeatureListBox.Items.RemoveAt(selectedIndex);
        }

        private void OnAddOtherFeatureButtonClick(Object sender, EventArgs e)
        {
            AddFeature(this.ProfileOtherFeatureTextBox.Text, this.ProfileOtherFeatureListBox.Items);
            this.ProfileOtherFeatureTextBox.Text = String.Empty;
        }

        private void OnRemoveOtherFeatureButtonClick(Object sender, EventArgs e)
        {
            Int32 selectedIndex;
            String feature;

            selectedIndex = this.ProfileOtherFeatureListBox.SelectedIndex;
            feature = this.ProfileOtherFeatureListBox.Items[selectedIndex].ToString();
            this.ProfileOtherFeatureListBox.Items.RemoveAt(selectedIndex);
            this.ProfileOtherFeatureTextBox.Text = feature;
        }

        private void OnProfileOtherFeatureListBoxSelectedIndexChanged(Object sender, EventArgs e)
        {
            this.RemoveOtherFeatureButton.Enabled = (ProfileOtherFeatureListBox.SelectedIndex != -1);
        }
    }
}